# -*- coding: utf-8 -*-
# Copyright (C) Softhealer Technologies.
from odoo import models


class Move(models.Model):
    _inherit = 'stock.move'


    def _sh_unreseve_qty(self):
        for move_line in self.sudo().move_line_ids:
            if move_line.product_id.type == 'consu':
                continue
            # unreserve qty
            quant = self.env['stock.quant'].sudo().search([('location_id', '=', move_line.location_id.id),
                                                           ('product_id', '=',
                                                            move_line.product_id.id),
                                                           ('lot_id', '=', move_line.lot_id.id)], limit=1)

            if quant:
                quant.write({'quantity': quant.quantity + move_line.qty_done})
            else:
                self.env['stock.quant'].sudo().create({'location_id': move_line.location_id.id,
                                                        'product_id': move_line.product_id.id,
                                                        'lot_id':move_line.lot_id.id,
                                                        'quantity':move_line.qty_done
                                                        })

            quant = self.env['stock.quant'].sudo().search([('location_id', '=', move_line.location_dest_id.id),
                                                           ('product_id', '=',
                                                            move_line.product_id.id),
                                                           ('lot_id', '=', move_line.lot_id.id)], limit=1)

            if quant:
                quant.write({'quantity': quant.quantity - move_line.qty_done})
            else:
                self.env['stock.quant'].sudo().create({'location_id': move_line.location_dest_id.id,
                                                        'product_id': move_line.product_id.id,
                                                        'lot_id':move_line.lot_id.id,
                                                        'quantity':move_line.qty_done * (-1)
                                                        })


class Production(models.Model):
    _inherit = 'mrp.production'

    def _check_stock_account_installed(self):
        stock_account_app = self.env['ir.module.module'].sudo().search(
            [('name', '=', 'stock_account')], limit=1)
        if stock_account_app.state != 'installed':
            return False
        else:
            return True
    def action_mrp_cancel(self):
        for rec in self:
            rec=rec.sudo()
            pickings = rec.picking_ids
            move_ids = (rec.move_raw_ids | rec.move_byproduct_ids | rec.move_dest_ids | rec.move_finished_ids | pickings.move_ids_without_package).sudo()
            move_lines = (move_ids.move_line_ids | rec.finished_move_line_ids).sudo()

            if move_ids:
                move_ids.write({'state': 'cancel'})
                move_lines.write({'state': 'cancel'})
                move_ids._sh_unreseve_qty()
                move_ids.stock_valuation_layer_ids.unlink()
            
            pickings.write({'state': 'cancel'})

            if rec.sudo().mapped('workorder_ids'):
                rec.sudo().mapped('workorder_ids').write({'state': 'cancel'})

            if rec._check_stock_account_installed():

                account_moves = move_ids.account_move_ids
                account_move_lines = account_moves.line_ids
                if account_move_lines:
                    reconcile_lines = self.env['account.partial.reconcile'].sudo().search(
                        ['|', ('credit_move_id', 'in', account_move_lines.ids), 
                        ('debit_move_id', 'in', account_move_lines.ids)])
                    if reconcile_lines:
                        reconcile_lines.sudo().unlink()
                    account_move_lines.analytic_line_ids.unlink()
                    account_moves.write({'state': 'draft', 'name': '/'})
                    account_moves.with_context(
                        {'force_delete': True}).unlink()

            rec.sudo().write({'state': 'cancel'})

    def action_mrp_cancel_draft(self):
        for rec in self:
            rec=rec.sudo()
            pickings = rec.picking_ids
            move_ids = (rec.move_raw_ids | rec.move_byproduct_ids | rec.move_dest_ids | rec.move_finished_ids | pickings.move_ids_without_package).sudo()
            move_lines = (move_ids.move_line_ids | rec.finished_move_line_ids).sudo()

            if move_ids:
                move_ids.write({'state': 'draft'})
                move_lines.write({'state': 'draft'})
                move_ids._sh_unreseve_qty()
                move_ids.stock_valuation_layer_ids.unlink()
            
            pickings.write({'state': 'draft'})

            if rec.sudo().mapped('workorder_ids'):
                rec.sudo().mapped('workorder_ids').write({'state': 'ready'})

            if rec._check_stock_account_installed():
                account_moves = move_ids.account_move_ids
                account_move_lines = account_moves.line_ids
                if account_move_lines:
                    reconcile_lines = self.env['account.partial.reconcile'].sudo().search(
                        ['|', ('credit_move_id', 'in', account_move_lines.ids), 
                        ('debit_move_id', 'in', account_move_lines.ids)])
                    if reconcile_lines:
                        reconcile_lines.sudo().unlink()
                    account_move_lines.analytic_line_ids.unlink()
                    account_moves.write({'state': 'draft', 'name': '/'})
                    account_moves.with_context(
                        {'force_delete': True}).unlink()

            rec.sudo().write({'state': 'draft'})

    def action_mrp_cancel_delete(self):
        for rec in self:
            rec=rec.sudo()
            pickings = rec.picking_ids
            move_ids = (rec.move_raw_ids | rec.move_byproduct_ids | rec.move_dest_ids | rec.move_finished_ids | pickings.move_ids_without_package).sudo()
            move_lines = (move_ids.move_line_ids | rec.finished_move_line_ids).sudo()

            if move_ids:
                move_ids.write({'state': 'draft'})
                move_lines.write({'state': 'draft'})
                move_ids._sh_unreseve_qty()
                move_ids.stock_valuation_layer_ids.unlink()
            
            pickings.write({'state': 'draft'})

            if rec.sudo().mapped('workorder_ids'):
                rec.sudo().mapped('workorder_ids').write({'state': 'ready'})

            if rec._check_stock_account_installed():
                account_moves = move_ids.account_move_ids
                account_move_lines = account_moves.line_ids
                if account_move_lines:
                    reconcile_lines = self.env['account.partial.reconcile'].sudo().search(
                        ['|', ('credit_move_id', 'in', account_move_lines.ids), 
                        ('debit_move_id', 'in', account_move_lines.ids)])
                    if reconcile_lines:
                        reconcile_lines.sudo().unlink()
                    account_move_lines.analytic_line_ids.unlink()
                    account_moves.write({'state': 'draft', 'name': '/'})
                    account_moves.with_context(
                        {'force_delete': True}).unlink()
            move_lines.unlink()
            move_ids.unlink()
            pickings.unlink()
            rec.sudo().write({'state': 'draft'})
            rec.sudo().unlink()

    def sh_cancel(self):
        if self.company_id.mrp_operation_type == 'cancel':
            self.action_mrp_cancel()
        elif self.company_id.mrp_operation_type == 'cancel_draft':
            self.action_mrp_cancel_draft()
        else:
            self.action_mrp_cancel_delete()
            return {
                'name': 'Manufacturing Orders',
                'type': 'ir.actions.act_window',
                'res_model': 'mrp.production',
                'view_type': 'form',
                'view_mode': 'tree,form',
                'target': 'current',
            }
